# Safe 'n Snug #

A simple application for backing up folders.

## License ##
Copyright (c) 2013 Zeff Svoboda

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

## How To ##

To use the program, clone the project from BitBucket, build the
AlteredConstants.SafeNSnug.sln solution, and run. (See Dependencies below for
important build information.)

Simply follow the steps to create a backup of any folder that you have read
access to. Backups are stored in "[My Documents]\Safe 'n Snug". To restore a
backup, select restore, select the backup you want to restore, then select the
location to restore the files to. (*Note:* the file structure inside the
original directory will be restored into selected folder.)

## Info ##

This project is currently in early beta. Zip compression is functional but the
encryption process is not yet implemented. (A functional prototype exists.)
I wasn't able to implement true TDD due to time constraints. Sever test cases
have been implemented, but the compression classes, for instance, would require
more interface abstraction of file system resources to truly test properly.

Some usability issues include the inability to (cleanly) cancel backup/restore
operations, file name conflicts are not handled well, the backup file selection
list could use more information/style, and a lot of error handle is absent
(most notably for file system operations).

The code base is also a bit messy and could use some more documentation but,
again, this was placed on the back burner in favor of a functional program.

Fortunately, the application should theoretically work well for large numbers
of files and large files. The use of the DotNetZip library with multithreading
and BackgroundWorkers to run the compression/decompression operations should
keep the program quick and responsive. Large files will stall the progress bar,
unfortunately--a sub-file-level polling system would have to be implemented in
order to improve this. (For the future encryption system, this will not be a
problem as polling happens at the byte-level.) The lack of cancellability also
reduces the current usefulness, but this is indeed an intended addition.

Planned encryption system will not be multi-threaded (beyond thread-separation
from the UI), but this could be implemented fairly easily by parallelizing
across file chunks.

Basically, I have grand plans that were felled by time constraints.

## Dependencies ##

This project has a few external dependencies. An MSBuild task exists to
automatically download these dependencies at build time **except** for the Page
Transition Control. See below for details.

Alternatively, the relevant versions of the individual dependencies can be
found on their respective project websites or downloaded manually through NuGet.

### Page Transition Control for WPF ###

This is a small library for doing page transitions in the UI. Originally, I
intended to implement this myself, but time ran out so I made use of this
helpful library.

This library is not available through NuGet and thus isn't downloaded during
build. It can be obtained and built from the project website or from the
downloads section of the Safe 'n Snug code repository.

Place the DLL in the lib directory in the UI project.

[Project Site](http://www.fyslexicduck.com/2011/05/page-transition-control-for-wpf.html)
[Binary Download](https://bitbucket.org/AlteredConstants/safe-n-snug/downloads/WpfPageTransitions.dll)

**Version 1.9**

### DotNetZip ###

DotNetZip handles all of the compression for the application. It provided a
powerful, concise, and multi-threaded implemenation for zip compression.

[Project Site](http://dotnetzip.codeplex.com/)

**Version 1.9**

### Ookii.Dialogs ###

The UI uses Ookii.Dialogs for folder selection dialogs. This library offered
the best folder browsing experience with graceful degradation to the XP-style
dialog when necessary.

[Project Site](http://www.ookii.org/software/dialogs/)

**Version 1.0**

### Machine.Specifications ###

Specifications designed for unit testing utilize the Machine.Specifications
library. Only needed for building the Specs project.

[Project Site](https://github.com/machine/machine.specifications)

**Version 0.5**

Follow the instructions on the website on how to use the test runner to verify
the specifications.

### Moq ###

Moq is used to mock out interfaces to do more precise unit testing. Only needed
for building the Specs project.

[Project Site](http://code.google.com/p/moq/)

**Version 4.0**

### Machine.Fakes ###

Machine.Fakes provides extra integration for Moq with Machine.Specifications.
Also requires the Machine.Fakes Moq adapter. Only needed for building the Specs
project.

[Project Site](https://github.com/machine/machine.fakes)

**Version 1.7**

### MSBuildVersioning ###

MSBuildVersioning simply includes the Mercurial changeset ID in the binary
version info fields for easy reference.

[Project Site](http://versioning.codeplex.com/)

**Version 1.0**
